/**
 * @file list-item组件
 * @author jiangzichen(jiangzichen01@baidu.com)
 */
/* globals Page, swan, getApp */
Component({ // eslint-disable-line
    externalClasses: ['custom-border', 'custom-main', 'custom-desc', 'custom-item'],
    properties: {
        key: {
            type: Number,
            value: 0,
            observer() {
                this.canChange = 1;
            }
        },
        index: {
            type: String,
            value: 0
        },
        titleType: {
            type: String,
            value: 'short'
        },
        className: {
            type: String,
            value: ''
        },
        main: {
            type: Object,
            value: {},
            observer() {
                if (this.canChange) {
                    this.renderMain();
                }
            }
        },
        desc: {
            type: Array,
            value: [],
            observer() {
                if (this.canChange) {
                    this.renderDesc();
                }
            }
        },
        avatar: {
            type: String,
            value: ''
        },
        icon: {
            type: String,
            value: ''
        },
        iconSize: {
            type: Number,
            value: 18
        },
        iconColor: {
            type: String,
            value: '#ccc'
        },
        showAction: {
            type: Boolean,
            value: false
        },
        showBorder: {
            type: Boolean,
            value: true
        }
    },
    data: {
        ready: false
    },
    attached() {
        this.renderData();
        swan.getSystemInfo({
            success: res => {
                if (res.platform === 'ios') {
                    this.setData({
                        isIos: 'ios'
                    });
                }
            }
        });
    },
    renderMain() {
        let {main = {}, cmain} = this.data;
        main = this.formItem(main);
        let params = {
            cmain: main
        };
        if (JSON.stringify(main) === JSON.stringify(cmain)) {
            return;
        }
        this.setData(params);
    },
    renderDesc() {
        let {desc = {}, cdesc} = this.data;
        desc = this.formItem(desc);
        let params = {
            cdesc: desc
        };
        if (JSON.stringify(desc) === JSON.stringify(cdesc)) {
            return;
        }
        this.setData(params);
    },
    renderData() {
        let {main = {}, desc = [], cmain, cdesc} = this.data;
        const arrow = this.defaultArrow();
        const hover = this.defaultHover();
        desc = !Array.isArray(desc) ? [desc] : desc;
        main = this.formItem(main);
        desc = desc.map(value => this.formItem(value));
        let params = {
            cmain: main,
            cdesc: desc,
            carrow: arrow,
            chover: hover,
            ready: true
        };
        if (JSON.stringify(main) === JSON.stringify(cmain) && JSON.stringify(desc) === JSON.stringify(cdesc)) {
            return;
        }
        this.setData(params, () => this.triggerEvent('ready'));
    },
    formItem(item) {
        let res = {};
        if (typeof item === 'string' || typeof item === 'number') {
            res.title = {
                content: item
            };
        }
        else {
            let {
                title = {},
                sub
            } = item;
            item.title = this.formText(title);
            // 非array类型转为array,array类型
            if (Array.isArray(sub)) {
                item.sub = item.sub.map(value => this.formText(value));
            }
            else {
                item.sub = [];
                sub && item.sub.push(this.formText(sub));
            }
            // 格式化字体大小单位转换为rpx字段
            if (item.sub && item.sub.length > 0) {
                item.sub.map(val => {
                    if (val.size) {
                        val.size = parseInt(val.size, 10) / 3 * 1.81 + 'rpx';
                    }
                    return val;
                });
            }
            res = item;
        }
        return res;
    },
    formText(item) {
        let res = item || {};
        if (typeof item === 'string' || typeof item === 'number') {
            let content = item;
            res = {
                content
            };
        }
        // fix不明map报错
        res.content = res.content && res.content.toString() || '';
        return res;
    },
    linkMap: ['nav-multi-line-item-1', 'nav-multi-line-item-2', 'nav-item-1', 'nav-item-2'],
    hoverMap: ['nav-multi-line-item-1', 'nav-multi-line-item-2', 'nav-multi-line-item-3', 'nav-item-1', 'nav-item-2'],
    defaultArrow() {
        const {
            arrow,
            type
        } = this.data;
        return arrow === undefined ? this.linkMap.includes(type) : arrow;
    },
    defaultHover() {
        const {
            hover,
            type
        } = this.data;
        return hover === undefined ? this.hoverMap.includes(type) : hover;
    },
    methods: {
        taphandle(e) {
            let {dataset = {}} = e.target;
            let that = e.currentTarget.dataset;
            const {index} = this.data;
            if (that.event) {
                this.triggerEvent(that.event, {
                    e,
                    index,
                    event: that.event,
                    info: that.info
                });
            }
            if (dataset.event) {
                this.triggerEvent(dataset.event, {
                    e,
                    index,
                    event: dataset.event,
                    info: dataset.info
                });
            }
            else {
                return;
            }
        }
    }
});
