/**
 * @file publish.js
 * @author houyu(houyu01@baidu.com)
 */
const api = require('./api');
let systemInfo = swan.getSystemInfoSync();
let isLogin = swan.isLoginSync();
// 工具方法，之后提取出来
const aopAfter = decorators => component => {
    for (let func in decorators) {
        const context = component[func] ? component : component.methods;
        const originMethod = context[func];
        if (originMethod) {
            context[func] = function (args) {
                const returnValue = originMethod.call(this, args);
                decorators[func]({
                    args,
                    thisObject: this,
                    returnValue
                });
            };
        }
    }
    return component;
};

// 具体要拦截的方法们
const contentDecorator = aopAfter({

    onTitleChange: ({thisObject}) => {
        thisObject.resolvePublichStatus();
    },

    onContentChange: ({thisObject}) => {
        thisObject.resolvePublichStatus();
    },

    delPicture: ({thisObject}) => {
        thisObject.resolvePublichStatus();
    },

    ready({thisObject}) {
        thisObject.resolvePublichStatus();
    }

});

/* globals Page, swan */

const Publish = contentDecorator({


    properties: {

        theme: {
            type: String,
            value: ''
        },

        title: {
            type: String,
            value: ''
        },

        titlePlaceHolder: {
            type: String,
            value: '输入标题'
        },

        content: {
            type: String,
            value: ''
        },

        contentPlaceHolder: {
            type: String,
            value: '输入内容'
        },

        pictureList: {
            type: Array,
            value: []
        },

        pictureLimit: {
            type: Number,
            value: 9
        },

        titleLimit: {
            type: Number,
            value: 20
        },

        titleMinLimit: {
            type: Number,
            value: -1
        },

        contentLimit: {
            type: Number,
            value: 120
        },

        contentMinLimit: {
            type: Number,
            value: -1
        },

        showPictureTips: {
            type: Boolean,
            value: false
        },

        pictureSizeLimit: {
            type: Number,
            value: 1e7
        },

        navigationBarTitle: {
            type: String,
            value: '发布'
        },

        tipsEdge: {
            type: Number,
            value: 20
        },

        pictureSelectLimit: {
            type: Number,
            value: 9
        },

        contentAutoHeight: {
            type: Boolean,
            value: true
        },

        contentHeight: {
            type: String,
            value: 'auto'
        },
        // 动态发布成功后的返回
        submitParam: {
            type: Object,
            value: {
                // 动态在开发者侧的唯一标识。
                // 当为空，默认开发者将数据托管在宿主，小程序侧创建动态后返回唯一标识给开发者。
                snid: '',
                // 动态的 schema 格式，跳转用
                path: '',
                // 小程序唯一标识
                appKey: '',
                // 小程序开发者唯一标识
                openid: ''
            },
            observer(value) {
                for (let item in value) {
                    if (!value[item]) {
                        return;
                    }
                }
                this.ugcCreate(value);
            }
        }
    },

    data: {
        disableStatus: 'disable',
        focusingClass: '',
        contentClass: '',
        title: '',
        content: '',
        pictureList: [],
        isLogin
    },

    contentInfos: {
        title: '',
        content: ''
    },

    lifetimes: {
        created() {

            if (swan.canIUse('swan.openCommunityEditor')) {
                swan.openCommunityEditor({
                    ...this.data,
                    contentPlaceholder: this.data.contentPlaceholder,
                    titlePlaceholder: this.data.titlePlaceholder,
                    imageConf: {
                        maxNum: this.data.pictureLimit,
                        ratio: 1
                    },
                    navBarTitleText: this.data.navigationBarTitle,
                    success(res) {

                        this.triggerEvent('chooseImage', {
                            images: res.tempFilePaths
                        });

                        this.triggerEvent('publish', {
                            title: res.title,
                            content: res.content
                        });
                    }
                });

                swan.navigateBack();
            }

            swan.setNavigationBarTitle({
                title: this.data.navigationBarTitle
            });

            !swan.getStorageSync('publishNum')
            && swan.setStorageSync('publishNum', 1);
        }
    },

    methods: {
        /**
         * 百度内部请求
         * @param {Object} [param] - 外部开发者返回的参数
         */
        ugcCreate(param) {
            let host = systemInfo.host;
            let {title, content, pictureList} = this.data;

            let innerData = {
                title,
                content,
                images: pictureList
            };
            let data = Object.assign(param, innerData);
            const link = 'https://ossapi.baidu.com/oss/publisher/ugc';
            let url = link + '?host_name=' + host + '&app_key=' + param.appKey;
            swan.request({
                url,
                header: {
                    'content-type': 'application/json'
                },
                method: 'POST',
                data,
                success: res => {
                    let successNum = swan.getStorageSync('publishNum');
                    if (res.data.errno === 0) {
                        if (successNum === 1) {
                            this.setData('showModal', true);
                        }
                        else {
                            swan.showToast({
                                icon: 'none',
                                title: '发布成功'
                            });
                        }
                        successNum++;
                        swan.setStorageSync('publishNum', successNum);
                    }
                    else {
                        swan.showToast({
                            icon: 'none',
                            title: res.data.msg || '发布失败'
                        });
                    }
                },
                fail: err => {
                    swan.showModal({
                        title: '标题',
                        content: err.errMsg,
                        showCancel: false
                    });
                }
            });
        },
        /**
         * 增加图片方法，并派发事件通知外部
         * @param {Object} [e] - 事件对象
         */
        addPicture(e) {
            let pictureList = this.data.pictureList || [];
            api
                .chooseImage({
                    count: this.data.pictureSelectLimit
                })
                .then(event => {
                    const selectedImages = event.tempFiles;
                    const totalImagesLength = selectedImages.length + this.data.pictureList.length;
                    if (totalImagesLength > this.data.pictureLimit) {
                        swan.showToast({
                            title: `图片总数超过${this.data.pictureLimit}张，无法上传`,
                            icon: 'none'
                        });
                        return;
                    }
                    // 有图片大于10M的话，则超标
                    if (this.data.pictureSizeLimit > 0
                        && selectedImages.some(file => file.size > this.data.pictureSizeLimit)
                    ) {
                        swan.showToast({
                            title: `部分图片大于${this.data.pictureSizeLimit / 1000000}M，无法上传`,
                            icon: 'none'
                        });
                        return Promise.reject();
                    }
                    this.triggerEvent('chooseImage', {
                        images: selectedImages.map(file => file.path)
                    });
                });
        },

        /**
         * 查看图片
         * @param {Object} [e] - 事件对象
         */
        viewPicture(e) {
            swan.previewImage({
                current: this.data.pictureList[e.currentTarget.dataset.index],
                urls: this.data.pictureList
            });
        },

        delPicture(e) {
            let pictureList = this.data.pictureList;
            let index = e.currentTarget.dataset.index;
            pictureList.splice(index, 1);
            this.triggerEvent('delImage', {index});
            this.setData({pictureList});
        },

        onSubmit(e) {
            if (!isLogin) {
                this.triggerEvent('onError', {
                    msg: 'NOT_LOGGED_IN',
                    isLogin
                });
                return;
            }
            const values = e.detail.value;
            if (this.data.disableStatus === 'disable') {
                return;
            }
            let openData = {
                title: values.title,
                content: values.content,
                images: this.data.pictureList
            };
            this.triggerEvent('publish', openData);
        },

        onTitleChange(e) {

            this.data.title = e.detail.value;
            let titleLen = this.data.title.length;

            if (titleLen >= this.data.titleLimit) {
                swan.showToast({
                    title: `标题最多${this.data.titleLimit}字`,
                    icon: 'none'
                });
            }

            this.setData({
                contentClass: titleLen > 0 ? 'has-content' : ''
            });
        },

        onContentChange(e) {
            this.data.content = e.detail.value;
        },

        /**
         * 处理"发布"按钮的展现策略
         *
         * @return {bool} - 是否禁用发布按钮
         */
        resolvePublichStatus() {
            let title = this.data.title || '';
            let content = this.data.content || '';
            let pictureList = this.data.pictureList || '';

            let hasTitle = !!title.length && !/^(\s|\r|\n)*$/.test(title);
            let hasContent = !!content.length && !/^(\s|\r|\n)*$/.test(content);
            let hasImage = !!pictureList.length;

            let contentOverMinLimit = content.length < this.data.contentMinLimit
                || content.length > this.data.contentLimit;
            let titleOverMinLimit = title.length < this.data.titleMinLimit
                || title.length > this.data.titleLimit;

            let isDisable = !((hasTitle && !titleOverMinLimit)
                              && ((hasContent && !contentOverMinLimit)
                                  || (hasImage && !~this.data.contentMinLimit)));

            this.setData({
                disableStatus: isDisable ? 'disable' : ''
            });

            return isDisable;
        },

        clearTitle() {
            this.setData({title: ''});
        },

        titleFocus() {
            this.setData({focusingClass: 'focus'});
        },

        titleBlur() {
            this.setData({focusingClass: ''});
        },

        closeModal() {
            this.setData('showModal', false);
        }
    }
});


Component(Publish); // eslint-disable-line
